'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});
exports.default = prepend;

var _utils = require('../utils');

/**
 * Resolves prepend option for css-modules-require-hook
 *
 * @param {*} value
 * @returns {Function}
 */
function prepend(value /* , currentConfig */) {
    let plugins = [];
    if (Array.isArray(value)) {
        for (let index = 0; index < value.length; index++) {
            const option = value[index];
            if ((0, _utils.isFunction)(option)) return plugins.push(option());

            const requiredOption = (0, _utils.requireLocalFileOrNodeModule)(option);
            if (!(0, _utils.isFunction)(requiredOption)) {
                throw new Error(`Configuration 'prepend[${index}]' module is not exporting a function`);
            }
            const returnedOptions = requiredOption();
            plugins = [...plugins, ...(returnedOptions.plugins || returnedOptions)];
        }
        return plugins;
    }

    if ((0, _utils.isString)(value)) {
        const requiredOption = (0, _utils.requireLocalFileOrNodeModule)(value);
        if (!(0, _utils.isFunction)(requiredOption)) {
            throw new Error(`Configuration 'prepend' module is not exporting a function`);
        }
        const returnedOptions = requiredOption();
        return returnedOptions.plugins || returnedOptions;
    }

    throw new Error(`Configuration 'prepend' is not an array`);
}